package yrpg.bamp.basedata.plugin.other.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.DBServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.HttpClientUtils;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 后台任务插件
 */
public class PushBeBankToMDMTask extends AbstractTask implements Plugin {

    public static final Log log= LogFactory.getLog(PushBeBankToMDMTask.class);

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        String url=System.getProperty("yrpg.fi.mdm.interbank.url");

        Map<String,String> header =new HashMap<>();
        header.put("Content-Type","application/json;charset=utf-8");

        int isfullget=Integer.valueOf(map.get("isfullget").toString());//是否全量  1 全量

        //获取昨天日期
        Date date=new Date();
        Calendar calendar=Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH,-1);
        Date yesterday=calendar.getTime();

        QFilter[] filters = new QFilter[]{};
        if(isfullget!=1){//非全量推送
            QFilter modifytimefi= new QFilter("createtime",">=",yesterday);
            filters = new QFilter[]{modifytimefi};
        }

        DynamicObjectCollection bebanks= QueryServiceHelper.query("bd_bebank","id,name,country.number,country.name,province.number,province.name,city.number,city.name,union_number,isoverdue,isfromcloud,provincecloud,citycloud,enable",filters);

        JSONObject paramsLogin = new JSONObject();
        if(bebanks.size()!=0){

            //分批推送，一次500条
            int count=bebanks.size();
            int num=count/500;
            if(count%500!=0) {
                num++;
            }
            for(int i=0;i<num;i++){
                JSONArray masterData = new JSONArray();
                if(i==num-1){
                    for(int j=500*i;j<bebanks.size();j++){
                        DynamicObject bebank=bebanks.get(j);
                        JSONObject data = new JSONObject();
                        long bankid=bebank.getLong("id");//id
                        String name=bebank.getString("name");//名称
                        String countrynumber=bebank.getString("country.number");//国家编码
                        String countryname=bebank.getString("country.name");//国家名称
                        String provincenumber=bebank.getString("province.number");//省份编码
                        String provincename=bebank.getString("province.name");//省份名称
                        String citynumber=bebank.getString("city.number");//城市编码
                        String cityname=bebank.getString("city.name");//城市名称
                        String unionnumber=bebank.getString("union_number");//联行号
                        String isoverdue=bebank.getString("isoverdue");//疑似过期  Y:是  N:否
                        Boolean isfromcloud=bebank.getBoolean("isfromcloud");//云端数据  true/false
                        String provincecloud=bebank.getString("provincecloud");//省份(云端)
                        String citycloud=bebank.getString("citycloud");//城市(云端)
                        String enable=bebank.getString("enable");//启用状态  0:禁用   1:可用
                        if("1".equals(enable)){
                            enable="启用";
                        }else{
                            enable="停用";
                        }

                        if(countrynumber==null){
                            countrynumber="";
                        }
                        if(countryname==null){
                            countryname="";
                        }
                        if(provincenumber==null){
                            provincenumber="";
                        }
                        if(provincename==null){
                            provincename="";
                        }
                        if(citynumber==null){
                            citynumber="";
                        }if(cityname==null){
                            cityname="";
                        }


                        data.put("id",String.valueOf(bankid));
                        data.put("bank_name",name);
                        data.put("region_code",countrynumber);
                        data.put("region_name",countryname);
                        data.put("province_code",provincenumber);
                        data.put("province_name",provincename);
                        data.put("city_code",citynumber);
                        data.put("city_name",cityname);
                        data.put("Interbank_number",unionnumber);
                        //data.put("Bank_category",);   银行类别名称 无
                        data.put("province_UnionPay",provincename);
                        data.put("city_UnionPay",cityname);
                        data.put("Suspected_expiration",isoverdue);
                        data.put("cloud_data",String.valueOf(isfromcloud));
                        data.put("province_cloud",provincecloud);
                        data.put("city_cloud",citycloud);
                        data.put("enablestate#name",enable);

                        masterData.add(data);
                    }
                }else{
                    for(int j=500*i;j<500*(i+1);j++){
                        DynamicObject bebank=bebanks.get(j);
                        JSONObject data = new JSONObject();
                        long bankid=bebank.getLong("id");//id
                        String name=bebank.getString("name");//名称
                        String countrynumber=bebank.getString("country.number");//国家编码
                        String countryname=bebank.getString("country.name");//国家名称
                        String provincenumber=bebank.getString("province.number");//省份编码
                        String provincename=bebank.getString("province.name");//省份名称
                        String citynumber=bebank.getString("city.number");//城市编码
                        String cityname=bebank.getString("city.name");//城市名称
                        String unionnumber=bebank.getString("union_number");//联行号
                        String isoverdue=bebank.getString("isoverdue");//疑似过期  Y:是  N:否
                        Boolean isfromcloud=bebank.getBoolean("isfromcloud");//云端数据  true/false
                        String provincecloud=bebank.getString("provincecloud");//省份(云端)
                        String citycloud=bebank.getString("citycloud");//城市(云端)
                        String enable=bebank.getString("enable");//启用状态  0:禁用   1:可用
                        if("1".equals(enable)){
                            enable="启用";
                        }else{
                            enable="停用";
                        }

                        if(countrynumber==null){
                            countrynumber="";
                        }
                        if(countryname==null){
                            countryname="";
                        }
                        if(provincenumber==null){
                            provincenumber="";
                        }
                        if(provincename==null){
                            provincename="";
                        }
                        if(citynumber==null){
                            citynumber="";
                        }if(cityname==null){
                            cityname="";
                        }


                        data.put("id",String.valueOf(bankid));
                        data.put("bank_name",name);
                        data.put("region_code",countrynumber);
                        data.put("region_name",countryname);
                        data.put("province_code",provincenumber);
                        data.put("province_name",provincename);
                        data.put("city_code",citynumber);
                        data.put("city_name",cityname);
                        data.put("Interbank_number",unionnumber);
                        //data.put("Bank_category",);   银行类别名称 无
                        data.put("province_UnionPay",provincename);
                        data.put("city_UnionPay",cityname);
                        data.put("Suspected_expiration",isoverdue);
                        data.put("cloud_data",String.valueOf(isfromcloud));
                        data.put("province_cloud",provincecloud);
                        data.put("city_cloud",citycloud);
                        data.put("enablestate#name",enable);

                        masterData.add(data);
                    }
                }

                paramsLogin.put("masterData",masterData);
                String reponseStrLogin="";
                long pkValue= DBServiceHelper.genGlobalLongId();

                Map<String, Object> logMap = new HashMap<>();
                logMap.put("opname", "定时任务推送联行号到MDM");
                logMap.put("thirdappname", "MDM");
                logMap.put("opdesc",url);
                logMap.put("request", paramsLogin.toString());
                logMap.put("status", "true");
                logMap.put("username", "yy");

                try{
                    reponseStrLogin= HttpClientUtils.postjson(url,header,paramsLogin.toString(),20000,40000);
                }catch (Exception e){
                    logMap.put("response", "调用MDM推送联行号接口失败！"+e.getMessage());
                    logMap.put("status", "false");
                    saveApiLog(logMap);
                }
                logMap.put("response",reponseStrLogin);
                if (!StringUtils.isEmpty(reponseStrLogin)) {
                    JSONObject responseJSON=null;
                    try {
                        responseJSON = JSONObject.parseObject(reponseStrLogin);
                    }catch (Exception e){
                        logMap.put("status", "false");
                        saveDetailApiLog(logMap,saveApiLog(logMap));
                    }
                    if(responseJSON.get("success")==null){
                        logMap.put("status", "false");
                        saveDetailApiLog(logMap,saveApiLog(logMap));
                    }
                    if (Boolean.valueOf(responseJSON.get("success").toString())) {
                        logMap.put("response","推送联行号至主数据成功！");
                        saveApiLog(logMap);
                    }else{
                        logMap.put("status", "false");
                        logMap.put("response", "调用MDM推送联行号接口返回异常！错误信息是"+responseJSON.get("message").toString());
                        saveDetailApiLog(logMap,saveApiLog(logMap));
                    }
                } else {
                    logMap.put("status", "false");
                    logMap.put("response", "接口返回值为空");
                    saveDetailApiLog(logMap,saveApiLog(logMap));
                }
            }
        }
    }

    public static long saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        ORM.create().setPrimaryKey(apiLog);
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

        return apiLog.getLong("id");

    }

    public static void saveDetailApiLog(Map<String,Object> logMap,long id){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_detail");
        //调用者
        apiLog.set("username",logMap.get("username"));
        //调用时间
        apiLog.set("opdate",new Date());
        //入参
        apiLog.set("reqpara_tag",logMap.get("request").toString());
        //响应
        apiLog.set("resppara_tag",logMap.get("response").toString());

        apiLog.set("id",id);

        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}